'use strict';

var React = require('react');
var reactCore = require('@patternfly/react-core');
var reactRouterDom = require('react-router-dom');
var dateFns = require('date-fns');
var reactRouterLastLocation = require('react-router-last-location');

function AppNavItem(_ref) {
  var title = _ref.title,
      to = _ref.to,
      exact = _ref.exact;

  if (!title || !to) {
    return React.createElement(reactCore.NavItemSeparator, {
      "data-testid": 'navitem-separator'
    });
  }

  return React.createElement(reactCore.NavItem, null, React.createElement(reactRouterDom.NavLink, {
    to: to,
    exact: exact,
    activeClassName: "pf-m-current"
  }, title));
}

var AppNavExpandable = function AppNavExpandable(_ref) {
  var title = _ref.title,
      to = _ref.to,
      items = _ref.items;
  var match = reactRouterDom.useRouteMatch({
    path: to
  });
  var isActive = !!match;
  return React.createElement(reactCore.NavExpandable, {
    title: title,
    isActive: isActive,
    isExpanded: isActive
  }, items.map(function (subNavItem, jdx) {
    return React.createElement(AppNavItem, Object.assign({}, subNavItem, {
      key: jdx
    }));
  }));
};

var AppNavGroup = function AppNavGroup(_ref) {
  var title = _ref.title,
      items = _ref.items;
  return React.createElement(reactCore.NavGroup, {
    title: title
  }, items.map(function (subNavItem, jdx) {
    return React.createElement(AppNavItem, Object.assign({}, subNavItem, {
      key: jdx
    }));
  }));
};

var AppLayoutContext =
/*#__PURE__*/
React.createContext({
  setBreadcrumb: function setBreadcrumb() {
    return void 0;
  }
});
var AppLayout = function AppLayout(_ref) {
  var logo = _ref.logo,
      logoProps = _ref.logoProps,
      _ref$navVariant = _ref.navVariant,
      navVariant = _ref$navVariant === void 0 ? 'horizontal' : _ref$navVariant,
      _ref$navItems = _ref.navItems,
      navItems = _ref$navItems === void 0 ? [] : _ref$navItems,
      _ref$navGroupsStyle = _ref.navGroupsStyle,
      navGroupsStyle = _ref$navGroupsStyle === void 0 ? 'grouped' : _ref$navGroupsStyle,
      headerTools = _ref.headerTools,
      _ref$startWithOpenNav = _ref.startWithOpenNav,
      startWithOpenNav = _ref$startWithOpenNav === void 0 ? true : _ref$startWithOpenNav,
      _ref$theme = _ref.theme,
      theme = _ref$theme === void 0 ? 'dark' : _ref$theme,
      _ref$mainContainerId = _ref.mainContainerId,
      mainContainerId = _ref$mainContainerId === void 0 ? 'main-container' : _ref$mainContainerId,
      children = _ref.children;

  var _React$useState = React.useState(startWithOpenNav),
      isNavOpen = _React$useState[0],
      setIsNavOpen = _React$useState[1];

  var _React$useState2 = React.useState(true),
      isMobileView = _React$useState2[0],
      setIsMobileView = _React$useState2[1];

  var _React$useState3 = React.useState(false),
      isNavOpenMobile = _React$useState3[0],
      setIsNavOpenMobile = _React$useState3[1];

  var _React$useState4 = React.useState(),
      breadcrumb = _React$useState4[0],
      setBreadcrumb = _React$useState4[1];

  var previousBreadcrumb = React.useRef();
  var handleSetBreadcrumb = React.useCallback(function (newBreadcrumb) {
    if (previousBreadcrumb.current !== newBreadcrumb) {
      previousBreadcrumb.current = newBreadcrumb;
      setBreadcrumb(previousBreadcrumb.current);
    }
  }, [setBreadcrumb, previousBreadcrumb]);
  var onNavToggleMobile = React.useCallback(function () {
    setIsNavOpenMobile(!isNavOpenMobile);
  }, [setIsNavOpenMobile, isNavOpenMobile]);
  var onNavToggle = React.useCallback(function () {
    setIsNavOpen(!isNavOpen);
  }, [setIsNavOpen, isNavOpen]);

  var onPageResize = function onPageResize(props) {
    setIsMobileView(props.mobileView);
  };

  React.useEffect(function () {
    setIsNavOpen(startWithOpenNav);
  }, [startWithOpenNav, setIsNavOpen]);
  var isVertical = navVariant === 'vertical';
  var variant = isVertical ? 'default' : 'horizontal';
  var Navigation = React.useMemo(function () {
    return navItems.length > 0 ? React.createElement(reactCore.Nav, {
      id: "nav-primary-simple",
      theme: theme,
      variant: variant
    }, React.createElement(reactCore.NavList, {
      id: "nav-list-simple"
    }, navItems.map(function (navItem, idx) {
      if (navItem && navItem.hasOwnProperty('items') && isVertical) {
        return navGroupsStyle === 'expandable' ? React.createElement(AppNavExpandable, Object.assign({}, navItem, {
          key: idx
        })) : React.createElement(AppNavGroup, Object.assign({}, navItem, {
          key: idx
        }));
      } else {
        return React.createElement(AppNavItem, Object.assign({}, navItem, {
          key: idx
        }));
      }
    }))) : null;
  }, [isVertical, navGroupsStyle, navItems, theme, variant]);
  var Header = React.useMemo(function () {
    return React.createElement(reactCore.PageHeader, {
      logo: logo,
      logoProps: logoProps,
      headerTools: headerTools,
      showNavToggle: isVertical,
      isNavOpen: isVertical ? isNavOpen : undefined,
      onNavToggle: isMobileView ? onNavToggleMobile : onNavToggle,
      topNav: isVertical ? undefined : Navigation
    });
  }, [logo, logoProps, headerTools, isVertical, isNavOpen, isMobileView, onNavToggleMobile, onNavToggle, Navigation]);
  var Sidebar = React.useMemo(function () {
    return navVariant === 'vertical' ? React.createElement(reactCore.PageSidebar, {
      nav: Navigation,
      isNavOpen: isMobileView ? isNavOpenMobile : isNavOpen,
      theme: theme,
      "data-testid": "app-sidebar"
    }) : undefined;
  }, [navVariant, Navigation, isMobileView, isNavOpenMobile, isNavOpen, theme]);
  return React.createElement(AppLayoutContext.Provider, {
    value: {
      setBreadcrumb: handleSetBreadcrumb
    }
  }, React.createElement(reactCore.Page, {
    mainContainerId: mainContainerId,
    header: Header,
    sidebar: Sidebar,
    breadcrumb: breadcrumb,
    onPageResize: onPageResize,
    skipToContent: React.createElement(reactCore.SkipToContent, {
      href: "#primary-app-container"
    }, "Skip to Content")
  }, children));
};

var FormatDate = function FormatDate(_ref) {
  var date = _ref.date,
      _ref$format = _ref.format,
      formatTpl = _ref$format === void 0 ? 'Pp' : _ref$format,
      options = _ref.options;
  date = typeof date === 'string' ? new Date(date) : date;
  return React.createElement(React.Fragment, null, dateFns.format(date, formatTpl, options));
};

var FormatDistance = function FormatDistance(_ref) {
  var date = _ref.date,
      _ref$base = _ref.base,
      base = _ref$base === void 0 ? new Date() : _ref$base,
      options = _ref.options;
  date = typeof date === 'string' ? new Date(date) : date;
  return React.createElement(React.Fragment, null, dateFns.formatDistance(date, base, options));
};

var FormatRelative = function FormatRelative(_ref) {
  var date = _ref.date,
      _ref$base = _ref.base,
      base = _ref$base === void 0 ? new Date() : _ref$base,
      options = _ref.options;
  date = typeof date === 'string' ? new Date(date) : date;
  return React.createElement(React.Fragment, null, dateFns.formatRelative(date, base, options));
};

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

var Loading = function Loading() {
  return React.createElement(reactCore.PageSection, {
    "aria-label": "Loading Content Container"
  }, React.createElement(reactCore.EmptyState, {
    variant: reactCore.EmptyStateVariant.full
  }, React.createElement(reactCore.EmptyStateIcon, {
    variant: "container",
    component: reactCore.Spinner
  }), React.createElement(reactCore.Title, {
    size: "lg",
    headingLevel: 'h1'
  }, "Loading")));
};

function LazyRoute(_ref) {
  var getComponent = _ref.getComponent,
      props = _objectWithoutPropertiesLoose(_ref, ["getComponent"]);

  var LazyComponent = React.useMemo(function () {
    return React.lazy(getComponent);
  }, [getComponent]);
  return React.createElement(reactRouterDom.Route, Object.assign({}, props), React.createElement(React.Suspense, {
    fallback: React.createElement(Loading, null)
  }, React.createElement(LazyComponent, null)));
}

function accessibleRouteChangeHandler(id, timeout) {
  if (timeout === void 0) {
    timeout = 50;
  }

  return window.setTimeout(function () {
    var mainContainer = document.getElementById(id);

    if (mainContainer) {
      mainContainer.focus();
    }
  }, timeout);
}
/**
 * a custom hook for sending focus to the primary content container
 * after a view has loaded so that subsequent press of tab key
 * sends focus directly to relevant content
 */

var useA11yRouteChange = function useA11yRouteChange(id) {
  if (id === void 0) {
    id = 'main-container';
  }

  var lastNavigation = reactRouterLastLocation.useLastLocation();
  var previousNavigation = React.useRef();
  React.useEffect(function () {
    var routeFocusTimer;
    var isStale = true;

    if (lastNavigation !== null) {
      previousNavigation.current = lastNavigation;
      isStale = false;
      routeFocusTimer = accessibleRouteChangeHandler(id, 50);
    }

    return function () {
      if (routeFocusTimer && isStale) {
        clearTimeout(routeFocusTimer);
      }
    };
  }, [id, lastNavigation, previousNavigation]);
};

function useDocumentTitle(title) {
  React.useEffect(function () {
    var originalTitle = document.title;
    document.title = title;
    return function () {
      document.title = originalTitle;
    };
  }, [title]);
}

var NotFound = function NotFound() {
  useA11yRouteChange();
  useDocumentTitle('Page not found');
  return React.createElement(reactCore.PageSection, null, React.createElement(reactCore.Alert, {
    variant: "danger",
    title: "404! This view hasn't been created yet."
  }), React.createElement("br", null), React.createElement(reactRouterDom.NavLink, {
    to: "/",
    className: "pf-c-nav__link"
  }, "Take me home"));
};

var SwitchWith404 = function SwitchWith404(_ref) {
  var children = _ref.children,
      props = _objectWithoutPropertiesLoose(_ref, ["children"]);

  var match = reactRouterDom.useRouteMatch();
  var defaultMatch = React.useMemo(function () {
    return match && React.createElement(reactRouterDom.Route, {
      path: match.path,
      exact: true
    });
  }, [match]);
  return React.createElement(reactRouterDom.Switch, Object.assign({}, props), children, defaultMatch, React.createElement(reactRouterDom.Route, null, React.createElement(NotFound, null)));
};

function useBreadcrumb(breadcrumb) {
  var context = React.useContext(AppLayoutContext);
  React.useEffect(function () {
    context.setBreadcrumb(breadcrumb);
    return function () {
      context.setBreadcrumb(null);
    };
  }, [context, breadcrumb]);
}

exports.AppLayout = AppLayout;
exports.AppLayoutContext = AppLayoutContext;
exports.AppNavExpandable = AppNavExpandable;
exports.AppNavGroup = AppNavGroup;
exports.AppNavItem = AppNavItem;
exports.FormatDate = FormatDate;
exports.FormatDistance = FormatDistance;
exports.FormatRelative = FormatRelative;
exports.LazyRoute = LazyRoute;
exports.Loading = Loading;
exports.NotFound = NotFound;
exports.SwitchWith404 = SwitchWith404;
exports.accessibleRouteChangeHandler = accessibleRouteChangeHandler;
exports.useA11yRouteChange = useA11yRouteChange;
exports.useBreadcrumb = useBreadcrumb;
exports.useDocumentTitle = useDocumentTitle;
//# sourceMappingURL=use-patternfly.cjs.development.js.map
