import { createElement, createContext, useState, useRef, useCallback, useEffect, useMemo, Fragment, lazy, Suspense, useContext } from 'react';
import { NavItemSeparator, NavItem, NavExpandable, NavGroup, Nav, NavList, PageHeader, PageSidebar, Page, SkipToContent, PageSection, EmptyState, EmptyStateVariant, EmptyStateIcon, Spinner, Title, Alert } from '@patternfly/react-core';
import { NavLink, useRouteMatch, Route, Switch } from 'react-router-dom';
import { format, formatDistance, formatRelative } from 'date-fns';
import { useLastLocation } from 'react-router-last-location';

function AppNavItem(_ref) {
  var title = _ref.title,
      to = _ref.to,
      exact = _ref.exact;

  if (!title || !to) {
    return createElement(NavItemSeparator, {
      "data-testid": 'navitem-separator'
    });
  }

  return createElement(NavItem, null, createElement(NavLink, {
    to: to,
    exact: exact,
    activeClassName: "pf-m-current"
  }, title));
}

var AppNavExpandable = function AppNavExpandable(_ref) {
  var title = _ref.title,
      to = _ref.to,
      items = _ref.items;
  var match = useRouteMatch({
    path: to
  });
  var isActive = !!match;
  return createElement(NavExpandable, {
    title: title,
    isActive: isActive,
    isExpanded: isActive
  }, items.map(function (subNavItem, jdx) {
    return createElement(AppNavItem, Object.assign({}, subNavItem, {
      key: jdx
    }));
  }));
};

var AppNavGroup = function AppNavGroup(_ref) {
  var title = _ref.title,
      items = _ref.items;
  return createElement(NavGroup, {
    title: title
  }, items.map(function (subNavItem, jdx) {
    return createElement(AppNavItem, Object.assign({}, subNavItem, {
      key: jdx
    }));
  }));
};

var AppLayoutContext =
/*#__PURE__*/
createContext({
  setBreadcrumb: function setBreadcrumb() {
    return void 0;
  }
});
var AppLayout = function AppLayout(_ref) {
  var logo = _ref.logo,
      logoProps = _ref.logoProps,
      _ref$navVariant = _ref.navVariant,
      navVariant = _ref$navVariant === void 0 ? 'horizontal' : _ref$navVariant,
      _ref$navItems = _ref.navItems,
      navItems = _ref$navItems === void 0 ? [] : _ref$navItems,
      _ref$navGroupsStyle = _ref.navGroupsStyle,
      navGroupsStyle = _ref$navGroupsStyle === void 0 ? 'grouped' : _ref$navGroupsStyle,
      headerTools = _ref.headerTools,
      _ref$startWithOpenNav = _ref.startWithOpenNav,
      startWithOpenNav = _ref$startWithOpenNav === void 0 ? true : _ref$startWithOpenNav,
      _ref$theme = _ref.theme,
      theme = _ref$theme === void 0 ? 'dark' : _ref$theme,
      _ref$mainContainerId = _ref.mainContainerId,
      mainContainerId = _ref$mainContainerId === void 0 ? 'main-container' : _ref$mainContainerId,
      children = _ref.children;

  var _React$useState = useState(startWithOpenNav),
      isNavOpen = _React$useState[0],
      setIsNavOpen = _React$useState[1];

  var _React$useState2 = useState(true),
      isMobileView = _React$useState2[0],
      setIsMobileView = _React$useState2[1];

  var _React$useState3 = useState(false),
      isNavOpenMobile = _React$useState3[0],
      setIsNavOpenMobile = _React$useState3[1];

  var _React$useState4 = useState(),
      breadcrumb = _React$useState4[0],
      setBreadcrumb = _React$useState4[1];

  var previousBreadcrumb = useRef();
  var handleSetBreadcrumb = useCallback(function (newBreadcrumb) {
    if (previousBreadcrumb.current !== newBreadcrumb) {
      previousBreadcrumb.current = newBreadcrumb;
      setBreadcrumb(previousBreadcrumb.current);
    }
  }, [setBreadcrumb, previousBreadcrumb]);
  var onNavToggleMobile = useCallback(function () {
    setIsNavOpenMobile(!isNavOpenMobile);
  }, [setIsNavOpenMobile, isNavOpenMobile]);
  var onNavToggle = useCallback(function () {
    setIsNavOpen(!isNavOpen);
  }, [setIsNavOpen, isNavOpen]);

  var onPageResize = function onPageResize(props) {
    setIsMobileView(props.mobileView);
  };

  useEffect(function () {
    setIsNavOpen(startWithOpenNav);
  }, [startWithOpenNav, setIsNavOpen]);
  var isVertical = navVariant === 'vertical';
  var variant = isVertical ? 'default' : 'horizontal';
  var Navigation = useMemo(function () {
    return navItems.length > 0 ? createElement(Nav, {
      id: "nav-primary-simple",
      theme: theme,
      variant: variant
    }, createElement(NavList, {
      id: "nav-list-simple"
    }, navItems.map(function (navItem, idx) {
      if (navItem && navItem.hasOwnProperty('items') && isVertical) {
        return navGroupsStyle === 'expandable' ? createElement(AppNavExpandable, Object.assign({}, navItem, {
          key: idx
        })) : createElement(AppNavGroup, Object.assign({}, navItem, {
          key: idx
        }));
      } else {
        return createElement(AppNavItem, Object.assign({}, navItem, {
          key: idx
        }));
      }
    }))) : null;
  }, [isVertical, navGroupsStyle, navItems, theme, variant]);
  var Header = useMemo(function () {
    return createElement(PageHeader, {
      logo: logo,
      logoProps: logoProps,
      headerTools: headerTools,
      showNavToggle: isVertical,
      isNavOpen: isVertical ? isNavOpen : undefined,
      onNavToggle: isMobileView ? onNavToggleMobile : onNavToggle,
      topNav: isVertical ? undefined : Navigation
    });
  }, [logo, logoProps, headerTools, isVertical, isNavOpen, isMobileView, onNavToggleMobile, onNavToggle, Navigation]);
  var Sidebar = useMemo(function () {
    return navVariant === 'vertical' ? createElement(PageSidebar, {
      nav: Navigation,
      isNavOpen: isMobileView ? isNavOpenMobile : isNavOpen,
      theme: theme,
      "data-testid": "app-sidebar"
    }) : undefined;
  }, [navVariant, Navigation, isMobileView, isNavOpenMobile, isNavOpen, theme]);
  return createElement(AppLayoutContext.Provider, {
    value: {
      setBreadcrumb: handleSetBreadcrumb
    }
  }, createElement(Page, {
    mainContainerId: mainContainerId,
    header: Header,
    sidebar: Sidebar,
    breadcrumb: breadcrumb,
    onPageResize: onPageResize,
    skipToContent: createElement(SkipToContent, {
      href: "#primary-app-container"
    }, "Skip to Content")
  }, children));
};

var FormatDate = function FormatDate(_ref) {
  var date = _ref.date,
      _ref$format = _ref.format,
      formatTpl = _ref$format === void 0 ? 'Pp' : _ref$format,
      options = _ref.options;
  date = typeof date === 'string' ? new Date(date) : date;
  return createElement(Fragment, null, format(date, formatTpl, options));
};

var FormatDistance = function FormatDistance(_ref) {
  var date = _ref.date,
      _ref$base = _ref.base,
      base = _ref$base === void 0 ? new Date() : _ref$base,
      options = _ref.options;
  date = typeof date === 'string' ? new Date(date) : date;
  return createElement(Fragment, null, formatDistance(date, base, options));
};

var FormatRelative = function FormatRelative(_ref) {
  var date = _ref.date,
      _ref$base = _ref.base,
      base = _ref$base === void 0 ? new Date() : _ref$base,
      options = _ref.options;
  date = typeof date === 'string' ? new Date(date) : date;
  return createElement(Fragment, null, formatRelative(date, base, options));
};

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

var Loading = function Loading() {
  return createElement(PageSection, {
    "aria-label": "Loading Content Container"
  }, createElement(EmptyState, {
    variant: EmptyStateVariant.full
  }, createElement(EmptyStateIcon, {
    variant: "container",
    component: Spinner
  }), createElement(Title, {
    size: "lg",
    headingLevel: 'h1'
  }, "Loading")));
};

function LazyRoute(_ref) {
  var getComponent = _ref.getComponent,
      props = _objectWithoutPropertiesLoose(_ref, ["getComponent"]);

  var LazyComponent = useMemo(function () {
    return lazy(getComponent);
  }, [getComponent]);
  return createElement(Route, Object.assign({}, props), createElement(Suspense, {
    fallback: createElement(Loading, null)
  }, createElement(LazyComponent, null)));
}

function accessibleRouteChangeHandler(id, timeout) {
  if (timeout === void 0) {
    timeout = 50;
  }

  return window.setTimeout(function () {
    var mainContainer = document.getElementById(id);

    if (mainContainer) {
      mainContainer.focus();
    }
  }, timeout);
}
/**
 * a custom hook for sending focus to the primary content container
 * after a view has loaded so that subsequent press of tab key
 * sends focus directly to relevant content
 */

var useA11yRouteChange = function useA11yRouteChange(id) {
  if (id === void 0) {
    id = 'main-container';
  }

  var lastNavigation = useLastLocation();
  var previousNavigation = useRef();
  useEffect(function () {
    var routeFocusTimer;
    var isStale = true;

    if (lastNavigation !== null) {
      previousNavigation.current = lastNavigation;
      isStale = false;
      routeFocusTimer = accessibleRouteChangeHandler(id, 50);
    }

    return function () {
      if (routeFocusTimer && isStale) {
        clearTimeout(routeFocusTimer);
      }
    };
  }, [id, lastNavigation, previousNavigation]);
};

function useDocumentTitle(title) {
  useEffect(function () {
    var originalTitle = document.title;
    document.title = title;
    return function () {
      document.title = originalTitle;
    };
  }, [title]);
}

var NotFound = function NotFound() {
  useA11yRouteChange();
  useDocumentTitle('Page not found');
  return createElement(PageSection, null, createElement(Alert, {
    variant: "danger",
    title: "404! This view hasn't been created yet."
  }), createElement("br", null), createElement(NavLink, {
    to: "/",
    className: "pf-c-nav__link"
  }, "Take me home"));
};

var SwitchWith404 = function SwitchWith404(_ref) {
  var children = _ref.children,
      props = _objectWithoutPropertiesLoose(_ref, ["children"]);

  var match = useRouteMatch();
  var defaultMatch = useMemo(function () {
    return match && createElement(Route, {
      path: match.path,
      exact: true
    });
  }, [match]);
  return createElement(Switch, Object.assign({}, props), children, defaultMatch, createElement(Route, null, createElement(NotFound, null)));
};

function useBreadcrumb(breadcrumb) {
  var context = useContext(AppLayoutContext);
  useEffect(function () {
    context.setBreadcrumb(breadcrumb);
    return function () {
      context.setBreadcrumb(null);
    };
  }, [context, breadcrumb]);
}

export { AppLayout, AppLayoutContext, AppNavExpandable, AppNavGroup, AppNavItem, FormatDate, FormatDistance, FormatRelative, LazyRoute, Loading, NotFound, SwitchWith404, accessibleRouteChangeHandler, useA11yRouteChange, useBreadcrumb, useDocumentTitle };
//# sourceMappingURL=use-patternfly.esm.js.map
